package me.lensfrex.manager.data.manager;

import me.lensfrex.manager.data.Student;
import me.lensfrex.manager.utils.DataLoader;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

/**
 * 对Student类的操作的底层封装，有最基础的操作
 */
public class ManagerTool {
    private final ArrayList<Student> students;
    private DataLoader dataLoader = new DataLoader();

    public ManagerTool() {
        this.students = dataLoader.load();
    }

    public ArrayList<Student> getStudentList() {
        return students;
    }

    /**
     * 添加一个学生
     *
     * @param id    学生id，long格式
     * @param name  学生姓名
     * @param score 学生成绩，map中key为科目名称，value为相应的成绩
     */
    protected void addStudent(long id, String name, Map<String, Integer> score) {
        Student student = new Student(id, name, score);
        students.add(student);

        dataLoader.write(students);
    }

    /**
     * 删除一个学生（按ID）
     *
     * @param id 学生id，long格式
     */
    protected void removeStudent(long id) {
        Student removedStudent = this.getStudent(id);

        if (removedStudent != null) {
            students.remove(removedStudent);
            System.out.println("Removed: " + id);
        }

        dataLoader.write(students);
    }

    /**
     * 删除一个学生（按姓名）
     *
     * @param name 学生姓名，long格式
     */
    protected void removeStudent(String name) {
        Student removedStudent = this.getStudent(name);

        if (removedStudent != null) {
            students.remove(removedStudent);
            System.out.println("Removed: " + name);
        }

        dataLoader.write(students);
    }

    /**
     * 按名字查找学生
     *
     * @param name 学生姓名
     * @return 学生的信息，若无该学生则返回null
     */
    protected Student getStudent(String name) {
        for (Student student : students) {
            if (student.getName().equals(name)) {
                return student;
            }
        }
        return null;
    }

    /**
     * 按学号查找学生
     *
     * @param id 学生学号
     * @return 学生信息，若无该学生则返回null
     */
    protected Student getStudent(long id) {
        for (Student student : students) {
            if (student.getId() == id) {
                return student;
            }
        }
        return null;
    }

    protected static final int SORT_COMPARE_BY_NAME = 0;
    protected static final int SORT_COMPARE_BY_ID = 1;
    protected static final int SORT_COMPARE_BY_GENERAL_SCORE = 2;

    /**
     * 对学生列表进行排序
     *
     * @param compareMethod 排序方式
     *                      可根据名字字典顺序（升序，0）、学生学号（升序，1）以及总成绩（降序，2）进行排序。
     * @return 排序后的学生列表
     */
    protected ArrayList<Student> getSortedStudentList(int compareMethod) {
        switch (compareMethod) {
            case SORT_COMPARE_BY_NAME:
                students.sort(Student.compareByName);
                break;
            case SORT_COMPARE_BY_ID:
                students.sort(Student.compareById);
                break;
            case SORT_COMPARE_BY_GENERAL_SCORE:
                students.sort(Student.compareByGeneralScore);
                break;
            default:
                break;
        }
        return students;
    }

    /**
     * 获取全班某课程的成绩信息
     *
     * @param subject 欲获取的科目信息
     * @return 包含成绩信息的HashMap，key为学生姓名，value为该科成绩数据
     */
    protected HashMap<String, Integer> getSubjectScores(String subject) {
        HashMap<String, Integer> scores = new HashMap<>();
        for (Student student : students) {
            scores.put(student.getName(), student.getScore().get(subject));
        }
        return scores;
    }

    /**
     * 获取全班所有同学的总成绩信息
     *
     * @return 包含全班所有同学成绩的HashMap，key为姓名，value为总成绩数据
     */
    protected HashMap<String, Integer> getAllStudentScores() {
        HashMap<String, Integer> result = new HashMap<>();

        int scoreSum = 0;
        for (Student student : students) {
            for (int score : student.getScore().values()) {
                scoreSum += score;
            }
            result.put(student.getName(), scoreSum);
            scoreSum = 0;
        }
        return result;
    }

    /**
     * 生成排名信息（按总分）
     *
     * @return 包含有排名信息的HashMap，key为姓名，value为相应的排名
     */
    protected HashMap<String, Integer> generateRanking() {
        HashMap<String, Integer> ranking = new HashMap<>();
        this.getSortedStudentList(SORT_COMPARE_BY_GENERAL_SCORE);

        for (Student student : students) {
            ranking.put(student.getName(), students.indexOf(student));
        }
        return ranking;
    }


    /**
     * 获取所有科目名称，若无学生信息则返回null
     *
     * @return 包含所有科目名称的Arraylist，若无学生信息则返回空的ArrayList
     */
    protected ArrayList<String> getAllSubjects() {
        if (students.isEmpty()) {
            return new ArrayList<>();
        }
        return new ArrayList<>(students.get(0).getScore().keySet());
    }

    /**
     * 获取学生总人数
     *
     * @return 学生总人数
     */
    protected int getStudentAmount() {
        return students.size();
    }

}
