package me.lensfrex.manager.data.manager;

import me.lensfrex.manager.data.Student;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * 对Student类的操作的上层封装，有较为多的操作
 */
public class StudentManager {
    private final ArrayList<Student> students;
    private final ManagerTool managerTool = new ManagerTool();

    public StudentManager() {
        this.students = managerTool.getStudentList();
    }

    public boolean hasStudent() {
        return !students.isEmpty();
    }

    /**
     * 增添学生。输入的字符串格式应为 [id] [name] ...[subjectName,score]
     * 如： 2333 LagSeeing English,100 Math,99 History,98
     *
     * @param input 待添加的学生信息
     */
    public void addStudent(String input) {
        String[] split = input.split(" ");
        try {
            long id = Long.parseLong(split[0]);
            String name = split[1];

            if (managerTool.getStudent(id) != null && managerTool.getStudent(name) != null) {
                return;
            }

            HashMap<String, Integer> score = new HashMap<>();
            String[] scoreInput;
            for (int i = 2; i < split.length; i++) {
                scoreInput = split[i].split(",");
                score.put(scoreInput[0], Integer.parseInt(scoreInput[1]));
            }

            managerTool.addStudent(id, name, score);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 删除一个学生（按ID）
     *
     * @param id 学生id，long格式
     */
    public void removeStudent(long id) {
        managerTool.removeStudent(id);
    }

    /**
     * 删除一个学生（按姓名）
     *
     * @param name 学生姓名，long格式
     */
    public void removeStudent(String name) {
        managerTool.removeStudent(name);
    }

    // 要是Java能像go语言那样返回两个返回值就好了...
    // 这样就能同时返回平均值和总值了...
    // 做宝可梦.jpg
    // 还是老老实实写两方法吧...

    /**
     * 获取班级某科目平均成绩
     *
     * @param subject 欲获取的科目
     * @return 科目平均成绩
     */
    public float getSubjectAvgScore(String subject) {
        return (float) getSubjectScore(subject) / managerTool.getStudentAmount();
    }

    /**
     * 获取班级某科目总成绩
     *
     * @param subject 欲获取的科目
     * @return 科目总成绩
     */
    public int getSubjectScore(String subject) {
        ArrayList<Integer> scores = new ArrayList<>(managerTool.getSubjectScores(subject).values());
        int sum = 0;
        for (int score : scores) {
            sum += score;
        }
        return sum;
    }

    /**
     * 获取每门科目的总成绩
     *
     * @return 包含每门课总成绩的HashMap，key为科目名称，value为该科目的总成绩
     */
    public HashMap<String, Integer> getAllSubjectScore() {
        ArrayList<String> subjects = managerTool.getAllSubjects();
        HashMap<String, Integer> scores = new HashMap<>();

        for (String subject : subjects) {
            scores.put(subject, this.getSubjectScore(subject));
        }

        return scores;
    }

    /**
     * 获取每门科目的平均成绩
     *
     * @return 包含每门课总成绩的HashMap，key为科目名称，value为该科目的平均成绩
     */
    public HashMap<String, Float> getAllSubjectAvgScore() {
        ArrayList<String> subjects = managerTool.getAllSubjects();
        HashMap<String, Float> scores = new HashMap<>();

        for (String subject : subjects) {
            scores.put(subject, this.getSubjectAvgScore(subject));
        }

        return scores;
    }

    /**
     * 获取某学生总成绩
     *
     * @param student 欲获取的学生
     * @return 该学生所有科目总成绩
     */
    public int getStudentGeneralScore(Student student) {
        return student.getGeneralScore();
    }

    /**
     * 获取所有科目
     *
     * @return 包含有科目名称的ArrayList
     */
    public ArrayList<String> getSubjects() {
        return managerTool.getAllSubjects();
    }

    /**
     * 按学生总分由高到低生成名次表
     *
     * @return 按顺序记录的名次表ArrayList
     */
    public ArrayList<String> generateRankingTable() {
        ArrayList<String> rankingTable = new ArrayList<>();
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_GENERAL_SCORE);

        for (int i = 0; i < sortStudentList.size(); i++) {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder
                    .append(i + 1).append("  |  ")
                    .append(sortStudentList.get(i).getName()).append("  |  ")
                    .append(students.get(i).getGeneralScore());

            rankingTable.add(stringBuilder.toString());
        }

        return rankingTable;
    }

    /**
     * 按学生学号由小到大生成成绩表
     *
     * @return 按学号顺序记录的成绩表ArrayList
     */
    public ArrayList<String> generateScoreTableById() {
        ArrayList<String> scoreTable = new ArrayList<>();
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_ID);

        for (Student student : sortStudentList) {
            scoreTable.add(this.getStudentInfo(student));
        }

        return scoreTable;
    }

    /**
     * 按学生姓名由小到大生成成绩表
     *
     * @return 按姓名顺序记录的成绩表ArrayList
     */
    public ArrayList<String> generateScoreTableByName() {
        ArrayList<String> scoreTable = new ArrayList<>();
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_NAME);

        for (Student student : sortStudentList) {
            scoreTable.add(this.getStudentInfo(student));
        }

        return scoreTable;
    }

    /**
     * 获取学生信息条目
     *
     * @param student 欲获取学生
     * @return 学生信息字符串
     */
    public String getStudentInfo(Student student) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder
                .append(student.getId()).append("  |  ")
                .append(student.getName()).append("  |  ")
                .append(this.getStudentGeneralScore(student)).append("  |  ")
                .append(listScore(student));

        return stringBuilder.toString();
    }

    /**
     * 列出某一学生的成绩
     *
     * @param student 学生
     * @return 成绩字符串
     */
    private String listScore(Student student) {
        StringBuilder stringBuilder = new StringBuilder();
        ArrayList<String> subjects = getSubjects();

        for (String subject : subjects) {
            stringBuilder
                    .append(subject).append(": ")
                    .append(student.getScore().get(subject)).append("  ");
        }
        return stringBuilder.toString();
    }

    /**
     * 列出所有科目的成绩信息，包括总分、平均分
     *
     * @return 含有科目成绩信息记录的ArrayList
     */
    public ArrayList<String> generateAllSubjectInfo() {
        ArrayList<String> subjects = this.getSubjects();
        ArrayList<String> record = new ArrayList<>();

        for (String subject : subjects) {
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder
                    .append(subject).append(" | General Score: ")
                    .append(this.getSubjectScore(subject)).append(" | Avg Score: ")
                    .append(this.getSubjectAvgScore(subject));
            record.add(stringBuilder.toString());
        }

        return record;
    }

    /**
     * 生成总的学生及科目成绩信息
     *
     * @return 信息字符串
     */
    public String generateAllInformation() {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder
                .append("------------------------------------\n")
                .append("      All Student Information       \n")
                .append("------------------------------------\n");

        ArrayList<String> studentTable = this.generateScoreTableById();
        for (String record : studentTable) {
            stringBuilder.append(record).append('\n');
        }

        stringBuilder
                .append("------------------------------------\n")
                .append("       All Subject Information      \n")
                .append("------------------------------------\n");

        ArrayList<String> subjectTable = this.generateAllSubjectInfo();
        for (String record : subjectTable) {
            stringBuilder.append(record).append('\n');
        }

        return stringBuilder.toString();
    }

    public ArrayList<Student> getStudents() {
        return students;
    }

    /**
     * 通过名字获取学生
     *
     * @param name 学生姓名
     * @return 学生类
     */
    public Student getStudent(String name) {
        return managerTool.getStudent(name);
    }

    /**
     * 通过学号获取学生
     *
     * @param id 学号
     * @return 学生类
     */
    public Student getStudent(long id) {
        return managerTool.getStudent(id);
    }

    /**
     * 获取某科目成绩分析HashMap
     *
     * @param subject 欲分析的科目
     * @return 成绩等级人数数组（长度为5，对应ABCDE五个等级）
     */
    private int[] getSubjectAnalysis(String subject) {
        ArrayList<Integer> scores = new ArrayList<>(managerTool.getSubjectScores(subject).values());
        int[] scoreLevelStudentNumber = new int[5];

        for (int score : scores) {
            scoreLevelStudentNumber[getScoreLevel(score)]++;
        }

        return scoreLevelStudentNumber;
    }

    /**
     * 获取所有成绩分析HashMap
     *
     * @return 包含有成绩等级人数的HashMap，key为科目名称，value为层次人数数组
     */
    public HashMap<String, int[]> getAllSubjectAnalysis() {
        ArrayList<String> subjects = managerTool.getAllSubjects();
        HashMap<String, int[]> analysis = new HashMap<>();

        for (String subject : subjects) {
            analysis.put(subject, this.getSubjectAnalysis(subject));
        }

        return analysis;
    }

    /**
     * 生成每科成绩层次分析(按百分制)的ArrayList
     * @return 包含成绩层次分析记录的ArrayList。每个层次等级数据为：人数/百分比
     */
    public ArrayList<String> generateSubjectAnalysis() {
        HashMap<String, int[]> analysis = getAllSubjectAnalysis();
        ArrayList<String> subjects = new ArrayList<>(analysis.keySet());

        ArrayList<String> records = new ArrayList<>();
        int studentAmount = managerTool.getStudentAmount();
        for (String subject : subjects) {
            int[] scoreLevelInfo = analysis.get(subject);

            StringBuilder stringBuilder = new StringBuilder().append(subject).append(":\n\t");
            stringBuilder
                    .append(String.format("Level A: %d/%.2f%% | ", scoreLevelInfo[SCORE_LEVEL_A], (float) scoreLevelInfo[SCORE_LEVEL_A]*100 / studentAmount))
                    .append(String.format("Level B: %d/%.2f%% | ", scoreLevelInfo[SCORE_LEVEL_B], (float) scoreLevelInfo[SCORE_LEVEL_B]*100 / studentAmount))
                    .append(String.format("Level C: %d/%.2f%% | ", scoreLevelInfo[SCORE_LEVEL_C], (float) scoreLevelInfo[SCORE_LEVEL_C]*100 / studentAmount))
                    .append(String.format("Level D: %d/%.2f%% | ", scoreLevelInfo[SCORE_LEVEL_D], (float) scoreLevelInfo[SCORE_LEVEL_D]*100 / studentAmount))
                    .append(String.format("Level E: %d/%.2f%%", scoreLevelInfo[SCORE_LEVEL_E], (float) scoreLevelInfo[SCORE_LEVEL_E]*100 / studentAmount));

            records.add(stringBuilder.toString());
        }

        return records;
    }

    private int getScoreLevel(int score) {
        if (score >= 90) {
            return SCORE_LEVEL_A;
        } else if (score >= 80) {
            return SCORE_LEVEL_B;
        } else if (score >= 70) {
            return SCORE_LEVEL_C;
        } else if (score >= 60) {
            return SCORE_LEVEL_D;
        } else {
            return SCORE_LEVEL_E;
        }
    }

    public static final int SCORE_LEVEL_A = 0;
    public static final int SCORE_LEVEL_B = 1;
    public static final int SCORE_LEVEL_C = 2;
    public static final int SCORE_LEVEL_D = 3;
    public static final int SCORE_LEVEL_E = 4;
}